(setf
 require-final-newline t
 version-control t
 delete-old-versions t      
 kept-old-versions 0
 kept-new-versions 4

 fill-column 62
 )

(require 'cc-mode)

(defun my-compile ()
  "Compile and switch to end of buffer"
  (interactive "")
  (compile "javac -g *.java")
  (other-window 1)
  (end-of-buffer))

(defun my-info ()
  "Info on my directory"
  (interactive "")
  (info "~/emacs/info/ben-dir"))

(defun my-list-buffers ()
  "Open *Buffer List* and put cursor in other window"
  (interactive "")
  (list-buffers)
  (other-window 1)
  (next-line 2))

(defun previous-buffer ()
  "Switch to previous buffer"
  (interactive)
  (switch-to-buffer (other-buffer)))

(push
 '("\\.cs$" . csharp-mode)
 auto-mode-alist)

(push
 '("\\.pad$" . tcl-mode)
 auto-mode-alist)

(push
 '("\\.c$" . c++-mode)
 auto-mode-alist)

(push
 '("\\.h$" . c++-mode)
 auto-mode-alist)

(defun incl-guard
    ()
  "write the include guard."
  (interactive)
  (save-excursion
    (goto-char 0)
    (let ((s
	   (upcase
	    (copy-sequence
	     (buffer-name)))))
      (dotimes (j (length s))
	(when (= ?. (aref s j))
	  (setf (aref s j) ?_)))
      (insert
       "#ifndef "
       s
       "
#define "
       s
       " 1

")
      (goto-char
       (point-max))
      (insert "#endif"))))

(defun switch-c-h
    ()
  "if you'rr in a .C, get the .h, and vv"
  (interactive)
  (let* ((p (copy-sequence
	     (buffer-name)))
	 (i (1- (length p)))
	 (c (aref p i)))
    (when (cond ((= ?C c)
		 (setf (aref p i) ?h))
		((= ?c c)
		 (setf (aref p i) ?h))
		((= ?h c)
		 (setf (aref p i) ?C)))
      (find-file p))))

(defun compile-c-file
    ()
  "Save buffers before compiling."
  (interactive)
  (save-some-buffers 4)

  (let ((p (buffer-name)))
    (compile
     (format "CC -c -g %s.cpp -I./../kpl/src -I/usr/local/include -DPAD_TK -DPAD_UNIX"
	     (substring p 0 (- (length p ) 4))))))

(defun compile-c-file-gcc
  ()
  "Save buffers before compiling."
  (interactive)
  (save-some-buffers 4)

  (let ((p (buffer-name)))
    (compile
     (format "g++ -c -g %s.C"
	     (substring p 0 (- (length p ) 2))))))


;;;
;;; C-x s ???
;;;
;;; . : .cshrc
;;; a : Address buffer
;;; b : Bib buffer
;;; B : Bugs buffer
;;; c : pad C files
;;; C : ChangeLog buffer
;;; d : directories
;;; e : emacs files
;;; f : ftp
;;; k : *compilation*
;;; R : RMAIL buffer
;;; T : ToDo buffer
;;; w : Word buffer
;;;

(defun select-thing
    (arg char)
  "Fast selection of stuff I use a lot."
  (interactive "p\ncBuffer Char: ")
  (case char
    
    (?.
     (find-file
      "~/.cshrc"))

    ;; Addresses     
    (?a
     (find-file
      "~/Address"))
     
    ;; Bibliography
    (?b
     (find-file
      "~/text/bib/bcr.bib"))
     
    ;; Bugs
    (?B
     (find-file
      "/local/bederson/pad_devo/Bugs"))
     
    ;; c files.
    (?c
     (cff
      "Pad C file: "
      '(
	(".cshrc" ."~/.cshrc")
	("Makefile" . "/local/bederson/pad_devo/src/Makefile")
	("bind" . "/local/bederson/pad_devo/src/bind.C")
	("cmd" . "/local/bederson/pad_devo/src/pad-tcl-cmd.C")
	("config" . "/local/bederson/pad_devo/src/pad-tcl-config.C")
	("defs" . "/local/bederson/pad_devo/src/defs.h")
	("gs" . "/local/bederson/pad_devo/src/gs.C")
	("html" . "/local/bederson/pad_devo/src/html.C")
	("image" . "/local/bederson/pad_devo/src/image.C")
	("init" . "/local/bederson/pad_devo/src/init.C")
	("line" . "/local/bederson/pad_devo/src/graph.C")
	("pad" . "/local/bederson/pad_devo/src/pad.C")
	("object" . "/local/bederson/pad_devo/src/object.C")
	("o2" . "/local/bederson/pad_devo/src/o2.C")
	("o3" . "/local/bederson/pad_devo/src/o3.C")
	("tcl" . "/local/bederson/pad_devo/src/pad-tcl.C")
	("text" . "/local/bederson/pad_devo/src/text.C")
	("tkpad" . "/local/bederson/pad_devo/src/tkpad.C")
	("veclib" . "/local/bederson/pad_devo/src/veclib.C")
	("view" . "/local/bederson/pad_devo/src/view.C")
	("widget" . "/local/bederson/pad_devo/src/widget.C")
	)))

    ;; ChangeLog
    (?C
     (find-file
      "/local/bederson/pad_devo/ChangeLog"))
     
     ;; dirs
     (?d
      (cdir
       "Dired of: "
       '(("bederson" . "~")
	 ("pad" . "~/pad")
	 ("pad_devo" . "/local/bederson/pad_devo")
	 ("emacs" . "~/emacs")
	 ("lemacs" . "~/emacs/lstead")
	 )))

     ;; emacs files
     (?e
      (cff
       "Emacs file: "
       '(("funs" . "~/emacs/funs.el")
	 ("bind" . "~/emacs/bind.el")
	 ("misc" . "~/emacs/misc.el")
	 ("version" . "~/emacs/version.el")
	 )))

     ;; ftp
     (?f
      (cff
       "Ftp to: "
       '(("gnu" . "/ftp@prep.mit.ai.edu:/pub/gnu")
	 ("elisp" . "/ftp@tut.cis.ohio-state.edu:/pub/gnu/emacs/elisp-archive")
	 )))
			  
    ;; Todo
    (?T
     (find-file
      "/local/bederson/pad_devo/ToDo"))
     
    ;; Words
    (?w
     (find-file
      "~/Words"))
     

     ;; Buffers.
     (?k (switch-to-buffer "*compilation*"))
     (?R (switch-to-buffer "RMAIL"))
     (beep)
     ))

(defun cff
    (prompt al)
  (find-file
   (cdr
    (assoc
     (completing-read
      prompt al nil t)
     al))))

(defun cdir
    (prompt al)
  (dired
   (cdr
    (assoc
     (completing-read prompt al nil t)
     al))))

(setq truncate-partial-width-windows nil)

(defun count-words-in-buffer
    ()
  (interactive)
  (count-words-in-region
   (point-min)
   (point-max)))

(defun count-words-in-region
    (min max)
  (interactive "r")
  (let ((real-buffer (current-buffer))
	(buffer (get-buffer-create "*Word Count*")))
    (set-buffer real-buffer)
    (call-process-region
     min max
     "wc"
     nil
     buffer
     nil)
    (set-buffer real-buffer)
    (princ (buffer-name) buffer)
    (princ " (region) as of " buffer)
    (princ (current-time-string) buffer)
    (terpri buffer)
    (pop-to-buffer buffer)
    (goto-char (point-min))
    (pop-to-buffer real-buffer)))
